package mods.ElectricRails;

import ic2.api.item.Items;

import java.util.logging.Logger;

import mods.ElectricRails.meta.Database;
import mods.ElectricRails.meta.ItemHost;
import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraftforge.common.Configuration;
import net.minecraftforge.oredict.OreDictionary;
import cpw.mods.fml.client.registry.RenderingRegistry;
import cpw.mods.fml.common.Loader;
import cpw.mods.fml.common.Mod;
import cpw.mods.fml.common.Mod.EventHandler;
import cpw.mods.fml.common.Mod.Instance;
import cpw.mods.fml.common.event.FMLInitializationEvent;
import cpw.mods.fml.common.event.FMLPostInitializationEvent;
import cpw.mods.fml.common.event.FMLPreInitializationEvent;
import cpw.mods.fml.common.network.NetworkMod;
import cpw.mods.fml.common.registry.GameRegistry;

@Mod(
        modid = ElectricRails.modid,
        name = ElectricRails.modid,
        version = "0.03e",
        dependencies = "required-after:IC2; after:Railcraft"
    )

@NetworkMod(
        clientSideRequired = true,
        serverSideRequired = true,
        channels = {PacketHandler.effectChannel},
        packetHandler = PacketHandler.class
    )

public class ElectricRails
{
    @Instance("ElectricRails")
    public static ElectricRails instance = null;

    // ============================== CONSTANTS ==============================

    public static final String modid = "ElectricRails";
    public static final String iconPrefix = modid + ":";
    public static final String RAILCRAFT = "Railcraft"; // RailCraft ModID (case sensitive)
    public static final Logger logger = Logger.getLogger(modid);

    // ======================== DEFAULT/CONFIG VALUES ========================

    // ===== ITEMS =====
    public int materialsItemID = 29899;

    // ===== TRACKS =====
    public int standardElectricTrackID = 781;
    public int advancedElectricTrackID = 782;
    public int thirdRailID             = 783;
    public int advancedTrackID         = 784;
    public int maglevRailID            = 785;
    public int maglevCoverID           = 786;

    // ===== STORAGE BLOCKS =====

    // ===== MACHINERY =====

    // ===== POWER PARAMATERS =====
    public double electricTracksEU = 1.0D; // EU/t
    public double thirdRailEU      = 1.0D;
    public double maglevRailEU     = 0.1D;

    // ===== SPEED AMPLIFIERS =====
    public double advancedSpeed    = 1.25; // xN
    public double maglevSpeed      = 3.0D; // xN

    // ===== DEBUG =====
    public boolean dumpOreNames = false;

    // =========================== COMMON OBJECTS ============================

    public CreativeTabs tab = null;
    public ItemHost materials = null;

    // ===== TRACKS =====

    public BlockRailEx blockStandardElectricTrack = null;
    public BlockRailEx blockAdvancedElectricTrack = null;
    public BlockRailEx blockAdvancedTrack = null;
    public BlockRailThird blockThirdRail = null;
    public BlockMaglevRail blockMaglevRail = null;
    public BlockMaglevCover blockMaglevCover = null;

    // ===== STORAGE BLOCKS =====

    // ===== VANILLA =====
    public ItemStack stackStoneSlab = new ItemStack(Block.stoneSingleSlab);
    public ItemStack stackStandardTrack = new ItemStack(Block.rail);

    @EventHandler
    public void preInit(FMLPreInitializationEvent event)
    {
        Configuration config = new Configuration(event.getSuggestedConfigurationFile());
        try
        {
            config.load();

            materialsItemID = config.get("Items", "ElectricRailID", materialsItemID).getInt(materialsItemID);

            standardElectricTrackID = config.getBlock("Blocks", "StandardElectricTrackID", standardElectricTrackID).getInt(standardElectricTrackID);
            advancedElectricTrackID = config.getBlock("Blocks", "AdvancedElectricTrackID", advancedElectricTrackID).getInt(advancedElectricTrackID);
            thirdRailID             = config.getBlock("Blocks", "ThirdRailID", thirdRailID).getInt(thirdRailID);
            advancedTrackID         = config.getBlock("Blocks", "AdvancedTrackID", advancedTrackID).getInt(advancedTrackID);
            maglevRailID            = config.getBlock("Blocks", "MaglevRailID", maglevRailID).getInt(maglevRailID);
            maglevCoverID           = config.getBlock("Blocks", "MaglevCoverID", maglevCoverID).getInt(maglevCoverID);

            config.addCustomCategoryComment("Energy", "Energy consumption rates when boosting cart (EU/t)");

            electricTracksEU = config.get("Energy", "ElectricTracksEU", electricTracksEU).getDouble(electricTracksEU);
            thirdRailEU      = config.get("Energy", "ThirdRailEU", thirdRailEU).getDouble(thirdRailEU);
            maglevRailEU     = config.get("Energy", "MaglevRailEU", maglevRailEU, "Amount of EU/t to consume per magnet rail block, e.g. it will be doubled in total but each part will be drained from different magnets.").getDouble(maglevRailEU);

            config.addCustomCategoryComment("Speed", "Max speed factors of tracks relative to standard tracks (1.0).");

            advancedSpeed    = config.get("Speed", "AdvancedTracksSpeed", advancedSpeed).getDouble(advancedSpeed);
            maglevSpeed      = config.get("Speed", "MaglevSpeed", maglevSpeed).getDouble(maglevSpeed);

            dumpOreNames = config.get("Debug", "DumpOreNames", dumpOreNames).getBoolean(dumpOreNames);
        }
        finally
        {
            if (config.hasChanged()) config.save();
        }
    }

    @EventHandler
    public void load(FMLInitializationEvent event)
    {
        if (event.getSide().isClient())
        {
            tab = new CreativeTabElectricRails("electricRails");
        }

        // Materials Item

        materials = new ItemHost(materialsItemID, Database.items);
        materials.setUnlocalizedName("erails.materials");
        materials.setCreativeTab(tab);
        GameRegistry.registerItem(materials, "erails.materials");

        // Standard Track

        blockStandardElectricTrack = new BlockRailEx(standardElectricTrackID, 1.0F, TileElectricRail.class);
        blockStandardElectricTrack.setUnlocalizedName("trackElectric");
        blockStandardElectricTrack.setTextureName(iconPrefix+"standard_electric_track");
        blockStandardElectricTrack.setHardness(0.7F);
        blockStandardElectricTrack.setStepSound(Block.soundMetalFootstep);
        blockStandardElectricTrack.setCreativeTab(tab);
        GameRegistry.registerBlock(blockStandardElectricTrack, "standardElectricTrack");

        // Advanced Track (non-electric)

        blockAdvancedTrack = new BlockRailEx(advancedTrackID, 1.25F, null);
        blockAdvancedTrack.setUnlocalizedName("trackAdvanced");
        blockAdvancedTrack.setTextureName(iconPrefix+"advanced_track");
        blockAdvancedTrack.setHardness(0.7F);
        blockAdvancedTrack.setStepSound(Block.soundMetalFootstep);
        blockAdvancedTrack.setCreativeTab(tab);
        GameRegistry.registerBlock(blockAdvancedTrack, "advancedTrack");

        // Advanced Electric Track

        blockAdvancedElectricTrack = new BlockRailEx(advancedElectricTrackID, 1.25F, TileElectricRail.class);
        blockAdvancedElectricTrack.setUnlocalizedName("trackAdvancedElectric");
        blockAdvancedElectricTrack.setTextureName(iconPrefix+"advanced_electric_track");
        blockAdvancedElectricTrack.setHardness(0.7F);
        blockAdvancedElectricTrack.setStepSound(Block.soundMetalFootstep);
        blockAdvancedElectricTrack.setCreativeTab(tab);
        GameRegistry.registerBlock(blockAdvancedElectricTrack, "advancedElectricTrack");

        // "Third Rail"

        blockThirdRail = new BlockRailThird(thirdRailID, Material.rock);
        blockThirdRail.setUnlocalizedName("thirdRail");
        blockThirdRail.setTextureName(iconPrefix+"third_rail");
        blockThirdRail.setHardness(0.7F);
        blockThirdRail.setStepSound(Block.soundStoneFootstep);
        blockThirdRail.setCreativeTab(tab);
        GameRegistry.registerBlock(blockThirdRail, "thirdRail");

        // Maglev Rail (magnet block)

        blockMaglevRail = new BlockMaglevRail(maglevRailID, Material.rock);
        blockMaglevRail.setUnlocalizedName("maglevRail");
        blockMaglevRail.setTextureName(iconPrefix+"maglev_rail");
        blockMaglevRail.setHardness(0.7F);
        blockMaglevRail.setStepSound(Block.soundStoneFootstep);
        blockMaglevRail.setCreativeTab(tab);
        GameRegistry.registerBlock(blockMaglevRail, "maglevRail");

        // Maglev Cover (physical rail)

        blockMaglevCover = new BlockMaglevCover(maglevCoverID, (float) maglevSpeed);
        blockMaglevCover.setUnlocalizedName("maglevCover");
        blockMaglevCover.setTextureName(iconPrefix+"maglev_cover");
        blockMaglevCover.setHardness(0.7F);
        blockMaglevCover.setStepSound(Block.soundStoneFootstep);
        blockMaglevCover.setCreativeTab(tab);
        GameRegistry.registerBlock(blockMaglevCover, "maglevCover");

        GameRegistry.registerTileEntity(TileElectricRail.class, "TileEntityElectricRail");

        GameRegistry.registerTileEntity(TileThirdRail.class, "TileEntityThirdRail");

        GameRegistry.registerTileEntity(TileMaglevCover.class, "TileEntityMaglevCover");
        GameRegistry.registerTileEntity(TileMaglevRail.class, "TileEntityMaglevRail");

        if (event.getSide().isClient())
        {
            int i = RenderingRegistry.getNextAvailableRenderId();
            RenderMaglevCover r = new RenderMaglevCover(i);
            RenderingRegistry.registerBlockHandler(r);
            blockMaglevCover.setRenderType(i);
        }
    }

    @EventHandler
    public void postInit(FMLPostInitializationEvent event)
    {
        if (dumpOreNames)
        {
            logger.info("Dumping ore names...");
            logger.info("===== START OF ORE NAMES LIST =====");
            String[] oreNames = OreDictionary.getOreNames();
            if (oreNames != null)
            {
                java.util.Arrays.sort(oreNames);
                for (String s: oreNames)
                {
                    if (s != null && !s.equals("")) logger.info("\"" + s + "\"");
                }
            }
            logger.info("===== END OF ORE NAMES LIST =====");
        }

        ItemStack stackIronIngot = new ItemStack(Item.ingotIron);
        ItemStack stackIronCable = Items.getItem("ironCableItem");

        ItemStack stackStandardRail  = GameRegistry.findItemStack(RAILCRAFT, "part.rail.standard", 1);
        ItemStack stackWoodenRailbed = GameRegistry.findItemStack(RAILCRAFT, "part.railbed.wood", 1);
        ItemStack stackStoneRailbed  = GameRegistry.findItemStack(RAILCRAFT, "part.railbed.stone", 1);

        // All Railcraft based recipes require that standard rails be available
        boolean isRailCraft = Loader.isModLoaded(RAILCRAFT) && stackStandardRail != null;

        // Electric Rail (RailCraft only)

        if (isRailCraft)
        {
            GameRegistry.addShapelessRecipe(
                    Database.electricRail.stack1,
                    new Object[] {
                        stackStandardRail,
                        stackIronCable
                        });
        }

        // Standard Electric Track

        if (isRailCraft && stackWoodenRailbed != null)
        {
            // RailCraft recipe that uses electric rails
            GameRegistry.addRecipe(
                    new ItemStack(blockStandardElectricTrack, 16),
                    new Object[] {
                        "R R", "RBR", "R R",
                        'R', Database.electricRail.stack1,
                        'B', stackWoodenRailbed
                        });
        }

        // Upgrade/crafting recipe for standard rails for both RailCraft and vanilla
        GameRegistry.addShapelessRecipe(
                new ItemStack(blockStandardElectricTrack, 2),
                new Object[] {
                    stackIronCable,
                    stackStandardTrack,
                    stackStandardTrack,
                    });

        // Advanced Track

        if (isRailCraft && stackStoneRailbed != null)
        {
            // RailCraft
            GameRegistry.addRecipe(
                    new ItemStack(blockAdvancedTrack, 16),
                    new Object[] {
                        "R R", "RBR", "R R",
                        'R', stackStandardRail,
                        'B', stackStoneRailbed
                        });
        }
        else
        {
            // Vanilla
            GameRegistry.addRecipe(
                    new ItemStack(blockAdvancedTrack, 16),
                    new Object[] {
                        "ISI", "ISI", "ISI",
                        'I', stackIronIngot,
                        'S', stackStoneSlab
                        });
        }

        // Advanced Electric Track Recipe

        if (isRailCraft && stackStoneRailbed != null)
        {
            // RailCraft
            GameRegistry.addRecipe(
                    new ItemStack(blockAdvancedElectricTrack, 16),
                    new Object[] {
                        "R R", "RBR", "R R",
                        'R', Database.electricRail.stack1,
                        'B', stackStoneRailbed
                        });
        }
        else
        {
            // Vanilla
            //GameRegistry.addRecipe(
            //        new ItemStack(blockAdvancedElectricTrack, 8),
            //        new Object[] {
            //            "ISI", "ISI", "ISI",
            //            'I', stackIronIngot,
            //            'S', stackStoneSlab
            //            });
        }

        // Upgrade/crafting recipe for advanced rails for both RailCraft and vanilla
        GameRegistry.addShapelessRecipe(
                new ItemStack(blockAdvancedElectricTrack, 2),
                new Object[] {
                    stackIronCable,
                    blockAdvancedTrack.stack1,
                    blockAdvancedTrack.stack1
                    });

        // "Third Rail"

        if (isRailCraft)
        {
            // RailCraft
            GameRegistry.addShapelessRecipe(
                    new ItemStack(blockThirdRail, 3),
                    new Object[] {
                        stackStoneSlab,
                        Database.electricRail.stack1
                        });
        }
        else
        {
            // Vanilla
            GameRegistry.addShapelessRecipe(
                    new ItemStack(blockThirdRail, 2),
                    new Object[] {
                        stackStoneSlab,
                        stackIronCable
                        });
        }

        // Maglev Rail (magnet block)

        GameRegistry.addRecipe(
                new ItemStack(blockMaglevRail, 6),
                new Object[] {
                    "SSS", "CCC", "SSS",
                    'S', new ItemStack(Block.stone),
                    'C', Items.getItem("coil")
                    });

        // Maglev Cover (physical rails)

        GameRegistry.addShapelessRecipe(
                new ItemStack(blockMaglevCover, 2),
                new Object[] {
                    stackStoneSlab,
                    new ItemStack(Block.sand)
                    });
    }

    /* for debug only
    public void dropTagList()
    {
        Logger log = Logger.getLogger(Loader.instance().activeModContainer().getModId());
        log.info("RailCraft tags:");
        String[] tags = TagList.getTags().toArray(new String[0]);
        Arrays.sort(tags);
        for (String s : tags) log.info("\"" + s + "\"");
    } */
}
