package mods.ElectricRails;

import ic2.api.energy.event.EnergyTileLoadEvent;
import ic2.api.energy.event.EnergyTileUnloadEvent;
import ic2.api.energy.tile.IEnergySink;

import java.util.Random;

import net.minecraft.nbt.NBTBase;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagDouble;
import net.minecraft.nbt.NBTTagInt;
import net.minecraft.tileentity.TileEntity;
import net.minecraftforge.common.ForgeDirection;
import net.minecraftforge.common.MinecraftForge;

public abstract class TileSink extends TileBase implements IEnergySink
{
    private static final double maxEnergy = 20.0D;
    public double energy = 0.0D;
    public boolean addedToEnergyNet = false;
    public Random rand = new Random();

    @Override
    public void updateEntity()
    {
        // Make sure that we connected to energy net
        if (!worldObj.isRemote && !addedToEnergyNet)
        {
            MinecraftForge.EVENT_BUS.post(new EnergyTileLoadEvent(this));
            addedToEnergyNet = true;
        }
    }

    public void removeFromEnergyNet()
    {
        if (addedToEnergyNet)
        {
            MinecraftForge.EVENT_BUS.post(new EnergyTileUnloadEvent(this));
            addedToEnergyNet = false;
        }
    }

    @Override
    public void invalidate()
    {
        removeFromEnergyNet();
        super.invalidate();
    }

    @Override
    public void onChunkUnload()
    {
        removeFromEnergyNet();
        super.onChunkUnload();
    }

    @Override
    public void readFromNBT(NBTTagCompound nbt)
    {
        super.readFromNBT(nbt);

        // Read "consumed" with backward compatibility
        double consumed = 0.0D;
        if (nbt.hasKey("consumed"))
        {
            NBTBase tag = nbt.getTag("consumed");
            if (tag instanceof NBTTagDouble)
            {
                consumed = ((NBTTagDouble) tag).data;
                nbt.removeTag("consumed");
            }
        }

        // Read "energy" key with backward compatibility
        if (nbt.hasKey("energy"))
        {
            NBTBase tag = nbt.getTag("energy");
            if (tag instanceof NBTTagInt)
            {
                energy = ((NBTTagInt) tag).data;
                nbt.removeTag("energy");

                energy -= consumed; // fix
                nbt.setDouble("energy", energy); // and write back
            }
            else if (tag instanceof NBTTagDouble)
            {
                energy = ((NBTTagDouble) tag).data;
            }
        }
    }

    @Override
    public void writeToNBT(NBTTagCompound nbt)
    {
        super.writeToNBT(nbt);
        nbt.setDouble("energy", energy);
    }

    @Override
    public boolean acceptsEnergyFrom(TileEntity emitter, ForgeDirection direction)
    {
        return direction == ForgeDirection.DOWN;
    }

    @Override
    public double demandedEnergyUnits()
    {
        double n = maxEnergy - energy;
        return n < 0.0D ? 0.0D : n;
    }

    @Override
    public double injectEnergyUnits(ForgeDirection directionFrom, double amount)
    {
        //// Check voltage
        //if (amount > getMaxSafeInput())
        //{
        //    onOvervoltage();
        //    return 0; // and eat all energy
        //}

        //int n = Math.min(getCapacity() - energy, amount);
        //energy += n;
        //return amount - n;
        energy += amount;
        return 0;
    }

    @Override
    public abstract int getMaxSafeInput();

    //public void onOvervoltage()
    //{
    //    invalidate();
    //    if (worldObj.isRemote)
    //    {
    //        // FX for client
    //        worldObj.playSoundEffect(xCoord + 0.5D, yCoord + 0.0625D, zCoord + 0.5D, "random.fizz", 0.5F, 2.6F + (worldObj.rand.nextFloat() - worldObj.rand.nextFloat()) * 0.8F);
    //        for (int i = 0; i < 8; i++)
    //        {
    //            worldObj.spawnParticle("largesmoke", xCoord + rand.nextGaussian(), yCoord, zCoord + rand.nextGaussian(), 0.0D, 0.0D, 0.0D);
    //        }
    //        doRandomSparks();
    //    }
    //    else
    //    {
    //        // Replace block with carrier, e.g. same block but without conductor(s)
    //        ItemStack carrier = getCarrier();
    //        if (carrier != null)
    //        {
    //            Block theBlock = Block.blocksList[carrier.itemID];
    //            if (theBlock != null && worldObj.setBlock(xCoord, yCoord, zCoord, carrier.itemID, carrier.getItemDamage(), 3) && worldObj.getBlockId(xCoord, yCoord, zCoord) == carrier.itemID)
    //            {
    //                // TODO Not sure that it is safe to pass NULL as player
    //                theBlock.onBlockPlacedBy(worldObj, xCoord, yCoord, zCoord, null, carrier);
    //                theBlock.onPostBlockPlaced(worldObj, xCoord, yCoord, zCoord, carrier.getItemDamage());
    //                return;
    //            }
    //        }
    //
    //        // Set to air if fail
    //        worldObj.setBlockToAir(xCoord, yCoord, zCoord);
    //    }
    //}
}
