package mods.ElectricRails.meta;

import java.util.List;

import mods.ElectricRails.ElectricRails;
import net.minecraft.client.renderer.texture.IconRegister;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.util.Icon;
import net.minecraftforge.oredict.OreDictionary;
import cpw.mods.fml.common.registry.GameRegistry;

public class ItemHost extends Item
{
    public final MetaItem[] metaItems;

    public ItemHost(int itemID, MetaItem[] metaItems)
    {
        super(itemID);
        setMaxDamage(0);
        setHasSubtypes(true);
        this.metaItems = metaItems;
        for (int i = 0; i < this.metaItems.length; i++)
        {
            MetaItem entry = this.metaItems[i];
            if (entry != null)
            {
                entry.stack1 = new ItemStack(this, 1, i); // create stock item stacks
                entry.stack4 = new ItemStack(this, 4, i);
                entry.stack9 = new ItemStack(this, 9, i);
                GameRegistry.registerCustomItemStack(entry.unlocalizedName, entry.stack1); // register custom itemstack
                OreDictionary.registerOre(entry.unlocalizedName, entry.stack1); // register in ore dictionary
            }
        }
    }

    @Override
    public String getUnlocalizedName(ItemStack stack)
    {
        if (stack != null)
        {
            int metadata = stack.getItemDamage();
            if (metadata >= 0 && metadata < metaItems.length)
            {
                MetaItem entry = metaItems[metadata];
                if (entry != null) return "item." + entry.unlocalizedName;
            }
        }
        return super.getUnlocalizedName(stack);
    }

    @Override
    public void registerIcons(IconRegister iconRegister)
    {
        for (int i = 0; i < metaItems.length; i++)
        {
            MetaItem entry = metaItems[i];
            if (entry != null)
            {
                entry.icon = iconRegister.registerIcon(ElectricRails.iconPrefix + entry.textureName);
            }
        }
    }

    @Override
    public Icon getIconFromDamage(int metadata)
    {
        if (metadata >= 0 && metadata < metaItems.length)
        {
            MetaItem entry = metaItems[metadata];
            if (entry != null) return entry.icon;
        }
        return null;
    }

    @Override
    public void getSubItems(int itemID, CreativeTabs tab, List list)
    {
        for (int i = 0; i < metaItems.length; i++)
        {
            MetaItem entry = metaItems[i];
            if (entry != null) list.add(entry.stack1);
        }
    }

    @Override
    public void addInformation(ItemStack stack, EntityPlayer player, List list, boolean shift)
    {
        if (stack != null)
        {
            int metadata = stack.getItemDamage();
            if (metadata >= 0 && metadata < metaItems.length)
            {
                MetaItem entry = metaItems[metadata];
                if (entry != null && entry.chemical != null && !entry.chemical.equals(""))
                {
                    list.add(entry.chemical);
                }
            }
        }
        super.addInformation(stack, player, list, shift);
    }
}
