package mods.ElectricRails.meta;

/**
 * Class for generating all derivatives for a material type.
 * Automatically add valid prefixes/postfixes to names.
 * Selecting of desired meta items is done by selective hosting them on a desired item.
 */
public class MetaMaterial
{
    public final int meltingPoint;
    public final MetaItem block, nugget, ingot, plate;
    public final MetaDust dust;

    /**
     * Full constructor.
     *
     * @param name - readable name, first letter should be capitalized
     * @param textureName - icon file name
     * @param chemical - chemical symbol
     * @param meltingPoint - melting point temperature of material in Kelvin (K).
     */
    public MetaMaterial(String name, String textureName, String chemical, int meltingPoint)
    {
        this.meltingPoint = meltingPoint;
        block  = new MetaItem("block"  + name, "block_" + textureName,  chemical);
        nugget = new MetaItem("nugget" + name, textureName + "_nugget", chemical);
        ingot  = new MetaItem("ingot"  + name, textureName + "_ingot",  chemical);
        plate  = new MetaItem("plate"  + name, textureName + "_plate",  chemical);
        dust   = new MetaDust(name, textureName, chemical);
    }

    /**
     * For rare earth elements.
     *
     * @param z - atomic number, just in case.
     * @param symbol - chemical symbol
     * @param name - readable name, first letter should be capitalized
     * @param meltingPoint - melting point temperature of material in Kelvin (K).
     */
    public MetaMaterial(int z, String symbol, String name, int meltingPoint)
    {
        this(name, name.toLowerCase(), symbol, meltingPoint);
    }
}
